@@ -3,35 +3,42 @@ import os
 import re
 import shlex
 import sys
-import warnings
+from contextlib import suppress
 
-from lintian_brush.control import (
+from debmutate._rules import (
+    Makefile,
+    check_cdbs,
+    dh_invoke_drop_argument,
+    dh_invoke_drop_with,
+    dh_invoke_replace_argument,
+    update_rules,
+)
+from debmutate.control import (
     ensure_exact_version,
     ensure_minimum_version,
     get_relation,
-    read_debian_compat_file,
-    ControlUpdater,
-    )
+)
+from debmutate.debhelper import (
+    MaintscriptEditor,
+    MaintscriptRemoveConffile,
+    read_debhelper_compat_file,
+)
+
 from lintian_brush.debhelper import (
+    DEBHELPER_BUILD_STEPS,
     detect_debhelper_buildsystem,
     lowest_non_deprecated_compat_level,
     maximum_debhelper_compat_version,
-    DEBHELPER_BUILD_STEPS,
-    )
+)
 from lintian_brush.fixer import (
     compat_release,
+    control,
     current_package_version,
+    fixed_lintian_tag,
+    is_debcargo_package,
     report_result,
-    )
-from lintian_brush.rules import (
-    check_cdbs,
-    dh_invoke_drop_with,
-    dh_invoke_drop_argument,
-    dh_invoke_replace_argument,
-    update_rules,
-    Makefile,
-    )
-
+    warn,
+)
 
 new_debhelper_compat_version = maximum_debhelper_compat_version(
     compat_release())
@@ -69,26 +76,25 @@ def autoreconf_disabled():
 
 
 if autoreconf_disabled():
-    try:
-        with open('configure', 'rb') as f:
-            for line in f:
-                if b'runstatedir' in line:
-                    break
-            else:
-                new_debhelper_compat_version = min(
-                    new_debhelper_compat_version, 10)
-                warnings.warn(
-                    'Not upgrading beyond debhelper %d, since the package '
-                    'disables autoreconf but its configure does not provide '
-                    '--runstatedir.' % new_debhelper_compat_version)
-    except FileNotFoundError:
-        pass
+    with suppress(IsADirectoryError, FileNotFoundError), \
+            open('configure', 'rb') as f:
+        for line in f:
+            if b'runstatedir' in line:
+                break
+        else:
+            new_debhelper_compat_version = min(
+                new_debhelper_compat_version, 10)
+            warn(
+                'Not upgrading beyond debhelper %d, since the package '
+                'disables autoreconf but its configure does not provide '
+                '--runstatedir.' % new_debhelper_compat_version)
 
 
 if os.path.exists('debian/compat'):
     # Package currently stores compat version in debian/compat..
 
-    current_debhelper_compat_version = read_debian_compat_file('debian/compat')
+    current_debhelper_compat_version = read_debhelper_compat_file(
+        'debian/compat')
     if current_debhelper_compat_version < new_debhelper_compat_version:
         with open('debian/compat', 'w') as cf:
             cf.write('%s\n' % new_debhelper_compat_version)
@@ -96,36 +102,44 @@ if os.path.exists('debian/compat'):
         # Nothing to do
         sys.exit(2)
 
-    with ControlUpdater() as updater:
+    with control as updater:
         updater.source["Build-Depends"] = ensure_minimum_version(
             updater.source.get("Build-Depends", ""),
             "debhelper",
             "%d~" % new_debhelper_compat_version)
 else:
-    # Assume that the compat version is set in Build-Depends
-    with ControlUpdater() as updater:
-        try:
-            offset, debhelper_compat_relation = get_relation(
-                updater.source.get("Build-Depends", ""), "debhelper-compat")
-        except KeyError:
-            sys.exit(2)
-        else:
-            if len(debhelper_compat_relation) > 1:
-                # Not sure how to deal with this..
+    try:
+        # Assume that the compat version is set in Build-Depends
+        with control as updater:
+            try:
+                offset, debhelper_compat_relation = get_relation(
+                    updater.source.get("Build-Depends", ""),
+                    "debhelper-compat")
+            except KeyError:
                 sys.exit(2)
-            if debhelper_compat_relation[0].version[0] != '=':
-                # Not sure how to deal with this..
-                sys.exit(2)
-            current_debhelper_compat_version = int(
-                debhelper_compat_relation[0].version[1])
-        if current_debhelper_compat_version < new_debhelper_compat_version:
-            updater.source["Build-Depends"] = ensure_exact_version(
-                    updater.source["Build-Depends"],
-                    "debhelper-compat",
-                    "%d" % new_debhelper_compat_version)
+            else:
+                if len(debhelper_compat_relation) > 1:
+                    # Not sure how to deal with this..
+                    sys.exit(2)
+                if debhelper_compat_relation[0].version[0] != '=':
+                    # Not sure how to deal with this..
+                    sys.exit(2)
+                current_debhelper_compat_version = int(
+                    debhelper_compat_relation[0].version[1])
+            if current_debhelper_compat_version < new_debhelper_compat_version:
+                updater.source["Build-Depends"] = ensure_exact_version(
+                        updater.source["Build-Depends"],
+                        "debhelper-compat",
+                        "%d" % new_debhelper_compat_version)
+    except FileNotFoundError:
+        # debcargo just uses the latest version and doesn't store debhelper
+        # version explicitly.
+        if is_debcargo_package():
+            sys.exit(0)
+        raise
 
 
-# For a list of behavior changes between debhelper compat verisons, see
+# For a list of behavior changes between debhelper compat versions, see
 # https://manpages.debian.org/testing/debhelper/debhelper.7.en.html#Supported_compatibility_levels
 
 
@@ -140,11 +154,8 @@ def line_matches_command(target, line, command):
     if line.startswith(command + b' ') or line == command:
         return True
 
-    if (target == (b'override_' + command) and
-            line.startswith(b'$(overridden_command)')):
-        return True
-
-    return False
+    return (target == (b'override_' + command) and
+            line.startswith(b'$(overridden_command)'))
 
 
 def update_line(line, orig, new, description):
@@ -173,7 +184,7 @@ def update_line_replace_argument(line, old, new, description):
     return line, False
 
 
-class PybuildUpgrader(object):
+class PybuildUpgrader:
 
     def __init__(self):
         # Does the dh line specify --buildsystem=pybuild?
@@ -202,6 +213,8 @@ class PybuildUpgrader(object):
                 buildsystem = detect_debhelper_buildsystem(step)
                 if buildsystem == 'python_distutils':
                     line += b' --buildsystem=pybuild'
+                    subitems.add(
+                        'Replace python_distutils buildsystem with pybuild.')
                     self.upgraded = True
             else:
                 if b'buildsystem=pybuild' in line:
@@ -214,6 +227,7 @@ class PybuildUpgrader(object):
             line, rest = line.split(b' -- ', 1)
             if step is None:
                 step = line[len(b'dh_auto_'):].split(b' ', 1)[0].decode()
+            subitems.add('Replace python_distutils buildsystem with pybuild.')
             line = (b'PYBUILD_' + step.upper().encode() + b'_ARGS=' +
                     shlex.quote(rest.decode()).encode() + b' ' + line)
 
@@ -228,7 +242,7 @@ def upgrade_to_dh_prep(line, target):
     return line
 
 
-class DhMissingUpgrader(object):
+class DhMissingUpgrader:
     """Replace --list-missing / --fail-missing with dh_missing."""
 
     def __init__(self):
@@ -315,6 +329,21 @@ def debhelper_argument_order(line, target):
     return line
 
 
+def override_dh_auto_test_drop_options(rule):
+    if b'override_dh_auto_test' not in rule.targets:
+        return
+    if (len(rule.lines) < 2
+            or rule.lines[1]
+            != b'ifeq (,$(filter nocheck,$(DEB_BUILD_OPTIONS)))'
+            or rule.lines[-1] != b'endif'):
+        return
+    subitems.add(
+        'Drop check for DEB_BUILD_OPTIONS containing "nocheck", '
+        'since debhelper now does this.')
+    del rule.lines[1]
+    del rule.lines[-1]
+
+
 def upgrade_to_debhelper_12():
 
     pybuild_upgrader = PybuildUpgrader()
@@ -356,14 +385,14 @@ def upgrade_to_debhelper_10():
     # dh_installinit will no longer install a file named debian/package as an
     # init script.
 
-    with ControlUpdater() as updater:
+    with control as updater:
         for binary in updater.binaries:
             name = binary['Package']
             if os.path.isfile(os.path.join('debian', name)):
                 os.rename(os.path.join('debian', name),
                           os.path.join('debian', name + '.init'))
                 subitems.add(
-                    'Rename debian/%s to debian/%s.init.' % (name, name))
+                    f'Rename debian/{name} to debian/{name}.init.')
 
 
 def upgrade_to_debhelper_11():
@@ -381,9 +410,11 @@ def upgrade_to_debhelper_11():
             package = service = parts[0]
         os.unlink(os.path.join('debian', name))
         subitems.add('Drop obsolete upstart file %s.' % name)
-        with open('debian/%s.maintscript' % package, 'a') as f:
-            f.write('rm_conffile /etc/init/%s.conf %s\n' % (
-                service, str(current_package_version())))
+        with MaintscriptEditor('debian/%s.maintscript' % package) as e:
+            e.append(MaintscriptRemoveConffile(
+                conffile='/etc/init/%s.conf' % service,
+                prior_version=current_package_version(),
+                package=None))
 
 
 def drop_dh_missing_fail(line, target):
@@ -393,8 +424,8 @@ def drop_dh_missing_fail(line, target):
 
     if not line.startswith(b'dh_missing '):
         return line
-    argv = shlex.split(line.decode())
-    for arg in ['--fail-missing', '-O--fail-missing']:
+    argv = line.split(b' ')
+    for arg in [b'--fail-missing', b'-O--fail-missing']:
         if arg in argv:
             # If we added the --fail-missing argument first, then
             # don't announce that here.
@@ -411,7 +442,7 @@ def drop_dh_missing_fail(line, target):
                     'debian/rules: Drop --fail-missing argument, '
                     'now the default.')
             argv.remove(arg)
-    return shlex.join(argv).encode()
+    return b' '.join(argv)
 
 
 def upgrade_to_debhelper_13():
@@ -419,26 +450,26 @@ def upgrade_to_debhelper_13():
     # dh_installtempfiles will handle d/tmpfile{,s}.  It prefer the latter
     # and warns about the former.
 
-    with ControlUpdater() as updater:
+    with control as updater:
         for binary in updater.binaries:
             name = binary['Package']
             tmpfile = os.path.join('debian', name + ".tmpfile")
             if os.path.isfile(tmpfile):
                 os.rename(tmpfile, tmpfile + 's')
                 subitems.add(
-                    'Rename %s to %ss.' % (tmpfile, tmpfile))
+                    f'Rename {tmpfile} to {tmpfile}s.')
         if os.path.isfile(os.path.join('debian', "tmpfile")):
             tmpfile = os.path.join('debian', "tmpfile")
             os.rename(tmpfile, tmpfile + 's')
             subitems.add(
-                'Rename %s to %ss.' % (tmpfile, tmpfile))
+                f'Rename {tmpfile} to {tmpfile}s.')
 
     update_rules([
         # The dh_missing command will now default to --fail-missing.  This can
         # be reverted to a non-fatal warning by explicitly passing
         # --list-missing like it was in compat 12.
         drop_dh_missing_fail,
-        ])
+        ], rule_cb=override_dh_auto_test_drop_options)
 
 
 upgrade_to_debhelper = {
@@ -451,23 +482,21 @@ upgrade_to_debhelper = {
 
 for version in range(int(str(current_debhelper_compat_version))+1,
                      int(str(new_debhelper_compat_version))+1):
-    try:
+    with suppress(KeyError):
         upgrade_to_debhelper[version]()
-    except KeyError:
-        pass
 
 if new_debhelper_compat_version > current_debhelper_compat_version:
     if current_debhelper_compat_version < lowest_non_deprecated_compat_level():
         kind = "deprecated"
-        tag = "package-uses-deprecated-debhelper-compat-version"
+        fixed_lintian_tag(
+            'source', "package-uses-deprecated-debhelper-compat-version",
+            info=f'{current_debhelper_compat_version}')
     else:
         kind = "old"
-        tag = "package-uses-old-debhelper-compat-version"
-    lines = ["Bump debhelper from %s %s to %s." % (
-        kind, current_debhelper_compat_version, new_debhelper_compat_version)]
-    for subitem in sorted(subitems):
-        lines.append("+ " + subitem)
+        fixed_lintian_tag(
+            'source', "package-uses-old-debhelper-compat-version",
+            info=f'{current_debhelper_compat_version}')
+    description = "Bump debhelper from {} {} to {}.".format(
+        kind, current_debhelper_compat_version, new_debhelper_compat_version)
 
-    report_result(
-        description='\n'.join(lines),
-        fixed_lintian_tags=[tag])
+    report_result(description=description, details=sorted(subitems))
