@@ -1,10 +1,11 @@
 #!/usr/bin/python3
 
-from lintian_brush.copyright import CopyrightUpdater, NotMachineReadableError
-from lintian_brush.fixer import report_result
-from lintian_brush.lintian_overrides import override_exists
-import sys
 import re
+import sys
+
+from debmutate.copyright import CopyrightEditor, NotMachineReadableError
+
+from lintian_brush.fixer import LintianIssue, report_result
 
 used = []
 defined = set()
@@ -19,8 +20,7 @@ def extract_licenses(synopsis):
     """
     ret = []
     for license in synopsis.split(" or "):
-        options = []
-        options.append(license)
+        options = [license]
         m = re.fullmatch(r'(.*) with (.*) exception', license)
         if m:
             license = m.group(1)
@@ -29,11 +29,11 @@ def extract_licenses(synopsis):
     return ret
 
 
-try:
-    with CopyrightUpdater() as updater:
-        if updater.copyright.header.license:
-            if updater.copyright.header.license.text:
-                defined.add(updater.copyright.header.license.synopsis)
+try:  # noqa: C901
+    with CopyrightEditor() as updater:
+        if (updater.copyright.header.license
+                and updater.copyright.header.license.text):
+            defined.add(updater.copyright.header.license.synopsis)
         for paragraph in updater.copyright.all_paragraphs():
             if not paragraph.license:
                 continue
@@ -87,20 +87,20 @@ try:
                     certainty = 'possible'
 
         if extra_defined and not extra_used:
-            for paragraph in list(updater.copyright._Copyright__paragraphs):
+            for paragraph in list(updater.copyright.all_paragraphs()):
                 if not paragraph.license:
                     continue
-                if override_exists(
-                        'unused-license-paragraph-in-dep5-copyright',
-                        package='source',
-                        info=paragraph.license.synopsis.lower()):
+                issue = LintianIssue(
+                    'source', 'unused-license-paragraph-in-dep5-copyright',
+                    info=paragraph.license.synopsis.lower())
+                if not issue.should_fix():
                     continue
                 if paragraph.license.synopsis in extra_defined:
-                    updater.copyright._Copyright__paragraphs.remove(paragraph)
+                    issue.report_fixed()
+                    updater.remove(paragraph)
 except (FileNotFoundError, NotMachineReadableError):
     pass
 else:
     report_result(
         "Remove unused license definitions for %s." % ', '.join(extra_defined),
-        fixed_lintian_tags=['unused-license-paragraph-in-dep5-copyright'],
         certainty=certainty)
